# Author(s) information
__authors__     = ['Sundar']
__author__      = ','.join(__authors__)
__credits__     = ['Ian Bruce', 'Lee']
__copyright__   = 'Copyright (c) 2014'
__license__     = 'GPL'

# Maintanence information
__maintainer__  = 'Sundar'
__email__       = 'feedback.multibootusb@gmail.com'
